/* =============================================================================
 *
 *  Description: This is a C implementation for Thread Hibernate
 *
 * -----------------------------------------------------------------------------
 *  Comments:
 *
 * ===========================================================================*/

/* Get access to any of the VDK features & datatypes used */
#include <sys/exception.h>
#include <sysreg.h>
#include <cdefBF533.h>
#include "Hibernate.h"

#define PF2_LED_CS	0x04		// SPI boot CS and LED toggle
#define SSYNC				asm ("ssync;")
#define NOP					asm ("nop;")
#define CLI					asm ("cli r0; idle;")


#define tristate_buffer_enable
#define RTC_wakeup

#define pFlashA_PortB_Data	(volatile unsigned char *)0x20270005

#define Ext_Reg_latch		(volatile unsigned short *)0x20100000
#define Dummy_Read			(volatile unsigned short *)0x20000000


#define	pSDRAM_PTR			(volatile unsigned long *)0x000000
#define	CLEAR_RTC_IRQS		0x807F		// Write-1-To-Clear all RTC interrupts

void Close_Peripherals(void);
void Save_System(void);
void Hibernate(void);
void Open_Peripherals(void);
void Hibernate2(void);

void
Hibernate_RunFunction(void **inPtr)
{
    /* Put the thread's "main" Initialization HERE */

    while (1)
    {
        /* Put the thread's "main" body HERE */
        VDK_PendSemaphore(kPushButton, 0);
        Close_Peripherals();
    	Save_System();
		Hibernate();
		Open_Peripherals();
		VDK_PostSemaphore(kBlink);
        /* Use a "break" instruction to exit the "while (1)" loop */
    }

    /* Put the thread's exit from "main" HERE */
    /* A thread is automatically Destroyed when it exits its run function */
}

int
Hibernate_ErrorFunction(void **inPtr)
{

    /* TODO - Put this thread's error handling code HERE */

      /* The default ErrorHandler goes to KernelPanic */

	VDK_CThread_Error(VDK_GetThreadID());
	return 0;
}

void
Hibernate_InitFunction(void **inPtr, VDK_ThreadCreationBlock *pTCB)
{
    /* Put code to be executed when this thread has just been created HERE */

    /* This routine does NOT run in new thread's context.  Any non-static thread
     *   initialization should be performed at the beginning of "Run()."
     */
}

void
Hibernate_DestroyFunction(void **inPtr)
{
    /* Put code to be executed when this thread is destroyed HERE */

    /* This routine does NOT run in the thread's context.  Any VDK API calls
     *   should be performed at the end of "Run()."
     */
}

/* ========================================================================== */
section("L1_code") void Hibernate2(void)
{
    long key = 0x12345678;
    int temp_irq_val, k;

    //Save key to location 0x3FEC
    *(pSDRAM_PTR+0xFFB) = key;
    //Save SP to location 0x3FF0
    *(pSDRAM_PTR+0xFFC) = sysreg_read(reg_SP);
    //Save FP to location 0x3FF4
    *(pSDRAM_PTR+0xFFD) = sysreg_read(reg_FP);
	//Save Hibernate Return Addres to location 0x3FFC
    *(pSDRAM_PTR+0xFFF) = sysreg_read(reg_RETS);

//	*pFIO_FLAG_T = PF2_LED_CS;		// Toggle the LED 30 times
//	while(1);
	
}



section ("L1_code") void Hibernate(void)
{
    
    long key = 0x12345678;
    int temp_irq_val, k;
	int delay=0x40000;
	
    //Save key to location 0x3FEC
    *(pSDRAM_PTR+0xFFB) = key;
    //Save SP to location 0x3FF0
    *(pSDRAM_PTR+0xFFC) = sysreg_read(reg_SP);
    //Save FP to location 0x3FF4
    *(pSDRAM_PTR+0xFFD) = sysreg_read(reg_FP);
	//Save Hibernate Return Addres to location 0x3FFC
    *(pSDRAM_PTR+0xFFF) = sysreg_read(reg_RETS);
        
	*pEBIU_SDGCTL |= SRFS;				// Enable self-refresh on SDRAM
	while(!(*pEBIU_SDSTAT & SDSRA));	// Wait For self refresh active

/*	
	temp_irq_val = cli();
	Configure_RTC();
	*pRTC_ISTAT = CLEAR_RTC_IRQS;	// Clear any pending IRQs
    *pSIC_IWR |= 0x80;				// Enable RTC wakeup
    idle();							// We should break out of this idle at the next RTC wakeup
    *pRTC_SWCNT = 0x0200;			// Reset Stopwatch for (80/32768) Seconds
    while (!(*pRTC_ISTAT & 0x8000));// Wait until all writes have completed
    *pRTC_ISTAT = CLEAR_RTC_IRQS;	// Acknowledge RTC Interrupt Request
    
*/	

	*pVR_CTL = (*pVR_CTL & 0xFFFC) | 0x8100;	// VROUT = b#00 For Hibernate, Enable SCKELOW
	



// This sequence MUST be performed in order for PLL_CTL and/or VR_CTL changes to take effect
// The Blackfin will go into Hibernate once the VR has been programmed. 
// Both the RTC wake-up and the #RESET wake-up will reboot the part.
	//temp_irq_val = cli();
//	idle();
//	idle();
//	CLI;
	//sti(temp_irq_val);

//   Write out to external latch so that SDRAM SDCKE signal is passed to SDRAM
//   Tristate buffer is active low.  Bit 10 of the EBIU databus, AMS1 CS. 
	
//	VDK_PushUnscheduledRegion();
//	VDK_PushCriticalRegion();
	
	VDK_PushUnscheduledRegion();
	VDK_PushCriticalRegion();
		
	*pEBIU_AMBCTL0	= 0xFFC2FFC2;  
	*pEBIU_AMBCTL1	= 0xFFC2FFC2;
	*pEBIU_AMGCTL	= 0x000f;
	SSYNC;
	
	
	
	
	*( (volatile unsigned short *)(0x20100000) ) = 0xFFD0;  // (JWS) 0XFBD0 = SDCKE is not tristated and core=0
															// (JWS) 0XFFD0 is SDCKE tristated	
//	NOP;
//	NOP;
//	NOP;															
															
	k = *(volatile unsigned short *)(0x20000000); //dummy read
	SSYNC;
/*
	while (delay--);	

// Set PF11 to latch output on high order GPIO latch to drive tristate buffer
    *pFIO_DIR       |= PF11;        // Set PF11 as output 
    *pFIO_FLAG_S    |= PF11;        // Set high 
	SSYNC;

	*( (volatile unsigned short *)(0x20100000) ) = 0xFBF0;  // (JWS) 0XFBD0 = SDCKE is not tristated and core=0
															// (JWS) 0XFFD0 is SDCKE tristated	
//	NOP;
//	NOP;
//	NOP;															
															
	k = *(volatile unsigned short *)(0x20000000); //dummy read
	SSYNC;	
	

	
		
//	*Ext_Reg_latch |= 0xFBF0;	// Write a 0 to 1 on SD5 (bit 6) to remove core power.
								// Also, tristate SDCKE to retain contents
//	k = *Dummy_Read;
//	k = *Ext_Reg_latch;			
//	SSYNC; 
	
	*pFIO_DIR       |= PF11;        // Set PF11 as output 
    *pFIO_FLAG_S    |= PF11;        // Set high 
	SSYNC;

	
//	idle();
//	idle();

	
*/
*pFIO_FLAG_T = PF2_LED_CS;		// Toggle the LED - if we get here something went wrong.

// Manually remove power from core!


	
	while(1);		
		
			
//	VDK_PopCriticalRegion();	
//	VDK_PopUnscheduledRegion();

		
	
//	while(1);  



   
/*
    long key = 0x12345678;
    int temp_irq_val, k;

    //Save key to location 0x3FEC
    *(pSDRAM_PTR+0xFFB) = key;
    //Save SP to location 0x3FF0
    *(pSDRAM_PTR+0xFFC) = sysreg_read(reg_SP);
    //Save FP to location 0x3FF4
    *(pSDRAM_PTR+0xFFD) = sysreg_read(reg_FP);
	//Save Hibernate Return Addres to location 0x3FFC
    *(pSDRAM_PTR+0xFFF) = sysreg_read(reg_RETS);
        
	*pEBIU_SDGCTL |= SRFS;				// Enable self-refresh on SDRAM
	while(!(*pEBIU_SDSTAT & SDSRA));	// Wait For self refresh active




	
#ifdef RTC_wakeup					// Added (JWS) since VROUT circuit does not work
// (JWS) I don't think this CLI is necessary now that we have removed the RTC stuff.		
	temp_irq_val = cli();
	Configure_RTC();
	*pRTC_ISTAT = CLEAR_RTC_IRQS;	// Clear any pending IRQs
    *pSIC_IWR |= 0x80;				// Enable RTC wakeup
    idle();							// We should break out of this idle at the next RTC wakeup
    *pRTC_SWCNT = 0x0200;			// Reset Stopwatch for (80/32768) Seconds
    while (!(*pRTC_ISTAT & 0x8000));// Wait until all writes have completed
    *pRTC_ISTAT = CLEAR_RTC_IRQS;	// Acknowledge RTC Interrupt Request
    
#endif
  


//#ifdef tristate_buffer_enable
	
// Added code below (JWS).  We must tristate SDCKE so that the SDRAM does not go out of self refresh 
//   during power down and boot up

//   Write out to external latch so that SDRAM SDCKE signal is passed to SDRAM
//   Tristate buffer is active low.  Bit 10 of the EBIU databus, AMS1 CS. 

	*( (volatile unsigned short *)(0x20100000) ) = 0xFFD0;  // (JWS) 0XFBD0 is SDCKE not tristated
															// (JWS) 0XFFD0 is SDCKE tristated	
	k = *(volatile unsigned short *)(0x20000000); //dummy read
	SSYNC;

// Set PF11 to latch output on high order GPIO latch to drive tristate buffer
    *pFIO_DIR       |= PF11;        // Set PF11 as output 
    *pFIO_FLAG_S    |= PF11;        // Set high 
	SSYNC;
	
//#endif

	*( (volatile unsigned short *)(0x20100000) ) = 0xFFD0;  // (JWS) 0XFBD0 is SDCKE not tristated
															// (JWS) 0XFFD0 is SDCKE tristated	
	k = *(volatile unsigned short *)(0x20000000); //dummy read
	SSYNC;

// Set PF11 to latch output on high order GPIO latch to drive tristate buffer
    *pFIO_DIR       |= PF11;        // Set PF11 as output 
    *pFIO_FLAG_S    |= PF11;        // Set high 
	SSYNC;





// (JWS) commented out for now  
//	*pFIO_FLAG_C = 0x80;			// set PF7 low for profiling purposes

// (JWS) This code is only used when Blackfin is using the internal regulator.
// Also, Bit 8 of VR_CTL allows the core to be reawaken via a RESET.
	*pVR_CTL = (*pVR_CTL & 0xFFFC) | 0x8100;	// VROUT = b#00 For Hibernate, Enable SCKELOW

// This sequence MUST be performed in order for PLL_CTL and/or VR_CTL changes to take effect
// The Blackfin will go into Hibernate once the VR has been programmed. 
// Both the RTC wake-up and the #RESET wake-up will reboot the part.
	//temp_irq_val = cli();
	idle();
	idle();
	//sti(temp_irq_val);

// (JWS) Added this to actually remove power from the core - external regulator   

	*pEBIU_AMBCTL0	= 0xFFC2FFC2;  
	*pEBIU_AMBCTL1	= 0xFFC2FFC2; // set timing for writing to GPIO registers on CS1 

//	*pEBIU_AMBCTL0	= 0xFFC2FbF0;  
//	*pEBIU_AMBCTL1	= 0x7bb07bb0; // set timing for writing to GPIO registers on CS1 
	*pEBIU_AMGCTL	= 0x000f;

	
//    idle();
//	idle();
	
//	VDK_PushUnscheduledRegion();	
//	k = *Ext_Reg_latch;	

//	k = *Dummy_Read;
	*Ext_Reg_latch |= 0xFFF0;	// Write a 0 to 1 on SD5 (bit 6) to remove core power.
	k = *Dummy_Read;
	SSYNC; 
//	VDK_PopUnscheduledRegion();
	k = *(volatile unsigned short *)(0x20100000); //dummy read	
	*( (volatile unsigned short *)(0x20100000) ) = 0xFFF0;	// Write a 0 to 1 on SD5 (bit 6) to remove core power.

//		*( (volatile unsigned short *)(0x20100000) ) = 0xFFFF;	// Write a 0 to 1 on SD5 (bit 6) to remove core power.
	SSYNC;
//	*( (volatile unsigned short *)(0x20100000) ) = 0xFFF0;	// Write a 0 to 1 on SD5 (bit 6) to remove core power.

	*( (volatile unsigned short *)(0x20100000) ) = 0xFBD0;  // (JWS) 0XFBD0 is SDCKE not tristated
															// (JWS) 0XFFD0 is SDCKE tristated	
	k = *(volatile unsigned short *)(0x20000000); //dummy read
	SSYNC;

// Set PF11 to latch output on high order GPIO latch to drive tristate buffer
    *pFIO_DIR       |= PF11;        // Set PF11 as output 
    *pFIO_FLAG_S    |= PF11;        // Set high 
	SSYNC;
	
	
		//SDRAM Refresh Rate Control Register
		*pEBIU_SDRRC = 0x00000817;	
		SSYNC;

		//SDRAM Memory Bank Control Register
		//  *pEBIU_SDBCTL = 0x00000011;  //16MByte 9 bit column addr 
		//	*pEBIU_SDBCTL = 0x00000013;  //32MByte 9 bit column addr
			*pEBIU_SDBCTL = 0x00000025;  //64MByte 10 bit column addr
		
		SSYNC;

		//SDRAM Memory Global Control Register	

		*pEBIU_SDGCTL = 0x1491998d;	// mobile sdram 0x1491998d

		SSYNC;	



	*pFIO_FLAG_T = PF2_LED_CS;		// Toggle the LED 30 times
	while(1);
*/
	
}
